from ai_traineree.agents.dqn import DQNAgent
from ai_traineree.env_runner import EnvRunner
from ai_traineree.tasks import GymTask
from ai_traineree.loggers import TensorboardLogger

import numpy as np
import pylab as plt
import torch


def run(
    reward_goal: float=100.0,
    max_episodes: int=2000,
    eps_start: float=1.0,
    eps_end: float=0.01,
    eps_decay: float=0.995,
):
    epsilon = eps_start
    mean_scores = []
    epsilons = []
    episode = 0

    while (episode < max_episodes):
        episode += 1
        score = 0
        state = task.reset()
        iterations = 0
        max_iterations = max_iterations if max_iterations is not None else max_iterations
        done = False

        while(iterations < max_iterations and not done):
            iterations += 1
            action = agent.act(state, eps)

            next_state, reward, done, _ = task.step(action)
            _rewards.append((iteration, reward))
            score += float(reward)

            agent.step(state, action, reward, next_state, done)
            # n -> n+1  => S(n) <- S(n+1)
            state = next_state

        mean_scores.append(sum(scores_window) / len(scores_window))
        epsilons.append(epsilon)

        epsilon = max(eps_end, eps_decay * epsilon)

        if mean_scores[-1] >= reward_goal and len(scores_window) == window_len:
            break

    return None


seed = 32167
# torch.set_deterministic(True)
torch.manual_seed(seed)
data_logger = TensorboardLogger()

env_name = 'CartPole-v1'
task = GymTask(env_name, seed=seed)
agent = DQNAgent(task.state_size, task.action_size, n_steps=5, seed=seed, device="cuda")
env_runner = EnvRunner(task, agent, data_logger=data_logger, seed=seed)

scores = env_runner.run(reward_goal=100, max_episodes=300, force_new=True, eps_decay=0.99, gif_every_episodes=5)
env_runner.interact_episode(render=True)

# plot the scores
fig = plt.figure()
ax = fig.add_subplot(111)
plt.plot(np.arange(len(scores)), scores)
plt.ylabel('Score')
plt.xlabel('Episode #')
plt.savefig(f'{env_name}.png', dpi=120)
plt.show()



from ai_traineree.agents import AgentBase
from ai_traineree.agents.dqn import DQNAgent
from ai_traineree.agents.ppo import PPOAgent
from ai_traineree.types import AgentState


class AgentFactory:

    @staticmethod
    def from_state(state: AgentState) -> AgentBase:
        if state.model == DQNAgent.name:
            return DQNAgent.from_state(state)
        elif state.model == PPOAgent.name:
            return PPOAgent.from_state(state)
        else:
            raise ValueError(f"Agent state contains unsupported model type: '{state.model}'")



import gym
import gym_pygame  # Import registers

from ai_traineree.agents.rainbow import RainbowAgent as Agent
from ai_traineree.env_runner import EnvRunner
from ai_traineree.loggers import TensorboardLogger
from ai_traineree.tasks import GymTask


env_name = 'Pixelcopter-PLE-v0'  # action=1 to start
# env_name = "FlappyBird-PLE-v0"  # Likely doesn't work because of assets
# env_name = "Pong-PLE-v0"  # action=2 to start
env = gym.make(env_name)
task = GymTask(env, seed=0)
task.hooks['post_reset_hook'] = lambda self: self.env.step(1)
data_logger = TensorboardLogger()

config = {
    'update_freq': 5,
}
agent = Agent(task.state_size, task.action_size, **config)
env_runner = EnvRunner(task, agent, data_logger=data_logger)

print('Action space:', env.action_space)
print('Obsevation space:', env.observation_space)
print('Obsevation space high:', env.observation_space.high)
print('Obsevation space low:', env.observation_space.low)

scores = env_runner.run(reward_goal=10, max_episodes=1000, eps_decay=0.99, force_new=True)
env_runner.interact_episode(render=True)
